//
//  ViewController.m
//  滤镜处理图片
//
//  Created by ly on 16/10/13.
//  Copyright © 2016年 ZST. All rights reserved.
//

#import "ViewController.h"
@interface ViewController ()<UIImagePickerControllerDelegate,UINavigationControllerDelegate>
{
    CIContext *context;
    CIFilter *filter;
    CIImage *beginImage;
    
    UIImageOrientation orientation;
}

@property (weak, nonatomic) IBOutlet UISlider *slider;

@property (weak, nonatomic) IBOutlet UIImageView *imageView;

@property (weak, nonatomic) IBOutlet UIImageView *oldImage;

@end


/*
 CIContext. 所有图像处理都是在一个CIContext 中完成的，这很像是一个Core Image处理器或是OpenGL的上下文。
 CIImage. 这个类保存图像数据。它可以从UIImage、图像文件、或者是像素数据中构造出来。
 CIFilter. 滤镜类包含一个字典结构，对各种滤镜定义了属于他们各自的属性。滤镜有很多种，比如鲜艳程度滤镜，色彩反转滤镜，剪裁滤镜等等
 */
@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    NSString * filePath = [[NSBundle mainBundle] pathForResource:@"tupian" ofType:@"jpg"];
    NSURL * urlPath = [NSURL fileURLWithPath:filePath ];
    beginImage = [CIImage imageWithContentsOfURL:urlPath];
    context = [CIContext contextWithOptions:nil]; // GPU 渲染,效率高,无法跨应用,home后停止执行,适合实时处理
    filter = [CIFilter filterWithName:@"CISepiaTone" keysAndValues:kCIInputImageKey, beginImage,@"inputIntensity",@0.8, nil];
    CIImage * outputImage = [filter outputImage];
    CGImageRef cgimg = [context createCGImage:outputImage fromRect:[outputImage extent]];
    _imageView.image = [UIImage imageWithCGImage:cgimg];
    CGImageRelease(cgimg);
 
    [self logAllFilters] ;
    
}


- (IBAction)change:(UISlider *)slider {
    
    [filter setValue:@(slider.value) forKey:@"inputIntensity"] ;
    CIImage * outputImage = [filter outputImage];
    CGImageRef cgimg = [context createCGImage:outputImage fromRect:[outputImage extent]];
    _imageView.image = [UIImage imageWithCGImage:cgimg scale:1.0 orientation:orientation];
    CGImageRelease(cgimg);
    
}

- (IBAction)selectPotos:(id)sender {
   UIImagePickerController * picker  = [[UIImagePickerController alloc] init];
    picker.delegate = self ;
    [self presentViewController:picker animated:YES completion:nil] ;
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info {
    
    [self dismissViewControllerAnimated:YES completion:nil];
    NSLog(@"%@",info) ;
    UIImage * originalImage = info[UIImagePickerControllerOriginalImage];
    orientation = originalImage.imageOrientation ;
    beginImage = [CIImage imageWithCGImage:originalImage.CGImage] ;
    [filter setValue:beginImage forKey:kCIInputImageKey];
    [self change:_slider] ;
    
    _oldImage.image = originalImage ;
}
- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)savePhoto:(id)sender {
    CIImage * saveImage = [filter outputImage];
    CIContext * saveContext = [CIContext contextWithOptions:@{kCIContextUseSoftwareRenderer : @YES}]; // CPU 渲染,效率低,可以跨应用,home后依然执行,适合保存图片
    CGImageRef cgimg = [saveContext createCGImage:saveImage fromRect:[saveImage extent]];
    _imageView.image = [UIImage imageWithCGImage:cgimg];

    UIImageWriteToSavedPhotosAlbum(self.imageView.image, self, @selector(imageSavedToPhotosAlbum:didFinishSavingWithError:contextInfo:), nil);

    CGImageRelease(cgimg);

    
}
- (void)imageSavedToPhotosAlbum:(UIImage *)image didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo {
    NSString *message = @"";
    if (!error) {
        message = @"成功保存到相册";
    }else
    {
        message = [error description];
    }
    NSLog(@"message is %@",message);
}

- (void) logAllFilters {
    NSArray * properties = [CIFilter filterNamesInCategory:kCICategoryBuiltIn];
    NSLog(@"%@",properties) ;
    
    for (NSString * filterName in properties) {
        CIFilter * fltr = [CIFilter filterWithName:filterName];
        NSLog(@"%@",[fltr attributes]) ;
    }
}



@end
